USE omop4;
GO

/* *** BLOCK 1: Extract relevant data element FROM source data and store in #tmp *** */
IF OBJECT_ID('tempdb..#tmp') IS NOT NULL
	DROP TABLE #tmp;
CREATE TABLE #tmp (
	person_id INT
	,eventDate DATE
	,eventConceptId VARCHAR(100)
	,eventType VARCHAR(100)
	,eventNumValue FLOAT NULL
	,eventStringValue VARCHAR(100) NULL
	);

INSERT INTO #tmp
SELECT DISTINCT *
FROM (
	SELECT P0.person_id
		,U.eventDate
		,U.eventConceptId
		,U.eventType
		,U.eventNumValue
		,U.eventStringValue
	FROM omop4.eMergeCohort P0
	LEFT JOIN (
		SELECT person_id AS person_id
		, CONVERT(varchar,year_of_birth)+'-'
						+ CONVERT(varchar, month_of_birth) +'-' 
						+CONVERT(varchar, day_of_birth)  AS eventDate
		,NULL AS eventConceptId
		,'Birthday' AS eventType
		,NULL AS eventNumValue
		,NULL AS eventStringValue
	    FROM omop4.dbo.person
	    
	    UNION ALL
		SELECT person_id AS person_id
			,condition_start_date AS eventDate
			,condition_concept_id AS eventConceptId
			,'DxHeartFailure' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.condition_occurrence
		WHERE condition_concept_id IN ('21001578','21001580','21001582','21001583','21001584','21001587','21001602','21001609','21001900','21013543','40248987','21001697','21001611','443580','443587','316139','319835','439846','35205240','35205243','141038','40479576','40479192','442310','444031','40480603','40481043','40481042','139036','40480602','35205257','35205242','35205241')
		
		UNION ALL
		SELECT person_id AS person_id
			,noteDate AS eventDate
			,NULL AS eventConceptId
			,'NoteHeartFailure' AS eventType
			,NULL AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.nlpNote
		WHERE AlgVar = 'heartFailure.problemList'
		
		UNION ALL
		SELECT person_id AS person_id
			,reportDate AS eventDate
			,NULL AS eventConceptId
			,'EchoEF' AS eventType
			,LVEF AS eventNumValue
		    ,NULL AS eventStringValue
		FROM omop4.dbo.nlpReportCardioEcho
		WHERE LVEF IS NOT NULL

   	
		    UNION ALL
        	SELECT person_id AS person_id
	     	 ,visit_start_date AS eventDate
	     	 , NULL AS eventConceptId
	     	 ,'Visit' AS eventType
	     	 ,NULL AS eventNumValue
			,NULL AS eventStringValue
	 		 FROM omop4.dbo.visit_occurrence
	  		
		
		) U ON P0.person_id = U.person_id
	) Utotal;

/* *** BLOCK Data Preparation: Need Age *** */
IF OBJECT_ID('tempdb..#tmpAge') IS NOT NULL
	DROP TABLE #tmpAge;
CREATE TABLE #tmpAge (
	person_id INT
	,birthDate DATE
	);
INSERT INTO #tmpAge
SELECT DISTINCT person_id
	,MIN(eventDate) AS birthDate
FROM #tmp
WHERE eventType = 'Birthday'
GROUP BY person_id
;

/* ***** To prepare: Continuous Enrollment Based on Visit ***** */
IF OBJECT_ID('tempdb..#tmpVisitYear') IS NOT NULL
	DROP TABLE #tmpVisitYear;
CREATE TABLE #tmpVisitYear (
	person_id INT NOT NULL
	,visitYear INT NOT NULL
	);
INSERT INTO #tmpVisitYear
SELECT distinct person_id, visitYear FROM (
SELECT person_id
, YEAR(eventDate) AS visitYear
, COUNT(eventDate) AS visitFreq
FROM #tmp T1
WHERE eventType = 'Visit'
AND DATEDIFF(year, 
		(SELECT birthDate
		FROM #tmpAge T2
		WHERE T1.person_id=T2.person_id)
		,eventDate) >=35
GROUP BY person_id, YEAR(eventDate) ) U
WHERE U.visitFreq >0 
;

/* *** BLOCK preparation: Get the first documented heart failure from rolling year *** */ 
IF OBJECT_ID('tempdb..#tmpHfFstDt') IS NOT NULL
	DROP TABLE #tmpHfFstDt;
CREATE TABLE #tmpHfFstDt (
	person_id INT NOT NULL
	,hfFstDt DATE NOT NULL
	);
INSERT INTO #tmpHfFstDt
SELECT W.person_id
,CASE WHEN  hfDxDt <hfNoteDt THEN hfDxDt ELSE hfNoteDt END AS hfFstDt
FROM (
  SELECT U.*, ROW_NUMBER() OVER(PARTITION BY person_id ORDER BY hfDxDt, hfNoteDt ASC) AS RowNumber
  FROM (
    SELECT T1.person_id, T1.eventDate AS hfDxDt, T2.eventDate AS hfNoteDt
    FROM #tmp T1
    LEFT JOIN #tmp T2
    ON T1.person_id=T2.person_id
    WHERE T1.eventType = 'DxHeartFailure' AND T2.eventType ='NoteHeartFailure'
  	    AND DATEDIFF(YEAR, T1.eventDate, T2.eventDate) >=-1 
	    AND DATEDIFF(YEAR, T1.eventDate, T2.eventDate) <=1 
  ) U
  WHERE U.hfDxDt IS NOT NULL AND U.hfNoteDt IS NOT NULL
) W
WHERE W.RowNumber=1
;	

/* *** BLOCK Preparation: get Ejection Fraction *** */
IF OBJECT_ID('tempdb..#tmpQualifiedEf') IS NOT NULL
	DROP TABLE #tmpQualifiedEf;
CREATE TABLE #tmpQualifiedEf (
	person_id INT NOT NULL
	,efDt DATE NOT NULL
	,efValue FLOAT
	,efPriority varchar(20)
	);
INSERT INTO #tmpQualifiedEf
SELECT T.person_id, T.eventDate AS efDt, eventNumValue AS efValue
,CASE WHEN DATEDIFF(DAY,(SELECT hfFstDt FROM #tmpHfFstDt WHERE #tmpHfFstDt.person_id = T.person_id), T.eventDate) >=0 
		AND DATEDIFF(DAY,(SELECT hfFstDt FROM #tmpHfFstDt WHERE #tmpHfFstDt.person_id = T.person_id), T.eventDate) <= 182 THEN 'priority1EF'
 WHEN DATEDIFF(DAY, T.eventDate, 
	(SELECT hfFstDt FROM #tmpHfFstDt WHERE #tmpHfFstDt.person_id = T.person_id)
	) >=0 AND DATEDIFF(DAY, T.eventDate, 
	(SELECT hfFstDt FROM #tmpHfFstDt WHERE #tmpHfFstDt.person_id = T.person_id)
	) <= 182 THEN 'priority2EF'
  WHEN DATEDIFF(DAY,(SELECT hfFstDt FROM #tmpHfFstDt WHERE #tmpHfFstDt.person_id = T.person_id), T.eventDate) >=183 
		AND DATEDIFF(DAY,(SELECT hfFstDt FROM #tmpHfFstDt WHERE #tmpHfFstDt.person_id = T.person_id), T.eventDate) <= 365 THEN 'priority3EF' 
  WHEN DATEDIFF(DAY, T.eventDate, 
	(SELECT hfFstDt FROM #tmpHfFstDt WHERE #tmpHfFstDt.person_id = T.person_id)
	) >=183 AND DATEDIFF(DAY, T.eventDate, 
	(SELECT hfFstDt FROM #tmpHfFstDt WHERE #tmpHfFstDt.person_id = T.person_id)
	) <= 365 THEN 'priority4EF' END AS efPriority
FROM #tmp T
WHERE eventType = 'EchoEF' AND eventNumValue IS NOT NULL;

/* *** BLOCK preparation: code heart failure type *** */
IF OBJECT_ID('tempdb..#tmpHfType') IS NOT NULL
	DROP TABLE #tmpHfType;
CREATE TABLE #tmpHfType (
	person_id INT NOT NULL
	,efDt DATE NOT NULL
	,efValue FLOAT
	,efPriority varchar(20)
	,RowNumber INT
	,hfType varchar(10)
	);
INSERT INTO #tmpHfType
SELECT J.*
	,CASE WHEN efValue IS NOT NULL AND efValue <50 THEN '1' 
	      WHEN efValue IS NOT NULL AND efValue >=50 THEN '2' 
		  END AS hfType
FROM (
	  SELECT *, ROW_NUMBER() OVER(PARTITION BY person_id ORDER BY efPriority ASC) AS RowNumber
	  FROM #tmpQualifiedEf
	  WHERE efPriority IS NOT NULL
	   ) J
	WHERE J.RowNumber =1
;


/* *** BLOCK 2: Get algorithm related variables *** */
/* split the AlgVar process, the running speed is faster */
IF OBJECT_ID('tempdb..#PreAlgVar1') IS NOT NULL
	DROP TABLE #PreAlgVar1;
CREATE TABLE #PreAlgVar1 (
	person_id INT NOT NULL
	,dxHeartFailureCnt INT NOT NULL
	,noteHeartFailureCnt INT NOT NULL
	,hfType varchar(10) NULL
	,efLT50Cnt INT NOT NULL
	);
INSERT INTO #PreAlgVar1
SELECT DISTINCT J.person_id
	,ISNULL(J0.dxHeartFailureCnt,0) AS dxHeartFailureCnt
	,ISNULL(J1.noteHeartFailureCnt,0) AS noteHeartFailureCnt
	,J2.hfType
	,ISNULL(J3.efLT50Cnt, 0) AS efLT50Cnt
FROM omop4.eMergeCohort J
LEFT JOIN (
	SELECT person_id
		,COUNT(DISTINCT eventDate) AS dxHeartFailureCnt
	FROM #tmp
	WHERE eventType = 'DxHeartFailure'
	GROUP BY person_id
	) J0 
ON J.person_id = J0.person_id
LEFT JOIN (
	SELECT person_id
		,COUNT(DISTINCT eventDate) AS noteHeartFailureCnt
	FROM #tmp
	WHERE eventType = 'NoteHeartFailure'
	GROUP BY person_id
	) J1 
ON J.person_id = J1.person_id
/* EF type */
LEFT JOIN (
	SELECT person_id
		,hfType
	FROM #tmpHfType
	) J2 
ON J.person_id = J2.person_id
/* for Control: about EF value */
LEFT JOIN (
	SELECT person_id
		,COUNT(DISTINCT eventDate) AS efLT50Cnt
	FROM #tmp
	WHERE eventType = 'EchoEF' AND eventNumValue < 50
	GROUP BY person_id
	) J3 
ON J.person_id = J3.person_id
;
IF OBJECT_ID('tempdb..#PreAlgVar2') IS NOT NULL
	DROP TABLE #PreAlgVar2;
CREATE TABLE #PreAlgVar2 (
	person_id INT NOT NULL
	,contEnrolCnt INT NOT NULL
	);
INSERT INTO #PreAlgVar2
/* for Control: continuous Enrollment */
SELECT DISTINCT J.person_id, ISNULL(J2.contEnrolCnt,0) AS contEnrolCnt
FROM omop4.eMergeCohort J WITH (NOLOCK)
LEFT JOIN (
SELECT U.person_id, COUNT (distinct U.startseat) AS contEnrolCnt
FROM (
	SELECT DISTINCT startseat = (
			SELECT top 1 a.visitYear
			FROM #tmpVisitYear a
			LEFT JOIN #tmpVisitYear b
				on a.visitYear = b.visitYear + 1
					and a.person_id = b.person_id
			WHERE b.visitYear + 1 is null
				and x1.visitYear > a.visitYear
				and a.person_id = x1.person_id
			ORDER BY visitYear desc
			)
		,x1.visitYear AS endseat
		,x1.person_id
	FROM (
		SELECT x.person_id
			,x.visitYear
		FROM #tmpVisitYear x
		LEFT JOIN #tmpVisitYear y
			on x.visitYear = y.visitYear + 1
				and x.person_id = y.person_id
		WHERE y.visitYear + 1 is not null
		) x1
	) U
WHERE (endseat - startseat) + 1 >=5
GROUP BY person_id ) J2
ON J.person_id = J2.person_id
;

/* get the #AlgVar */
IF OBJECT_ID('tempdb..#AlgVar') IS NOT NULL
	DROP TABLE #AlgVar;
CREATE TABLE #AlgVar (
	person_id INT NOT NULL
	,diseaseName VARCHAR(50) NOT NULL
	,caseControl VARCHAR(50) NOT NULL
	,C00 INT NOT NULL
	,C01 INT NOT NULL
	,C02 varchar(10) NULL
	,C03 INT NOT NULL
	,C04 INT NOT NULL
	);
INSERT INTO #AlgVar
SELECT DISTINCT J.person_id
	,'HeartFailure' AS diseaseName
	,'CASE/CONTROL' AS caseControl
	, J1.dxHeartFailureCnt AS C00
	,J1.noteHeartFailureCnt AS C01
	,J1.hfType AS C02
	,J1.efLT50Cnt AS C03
	,J2.contEnrolCnt AS C04
FROM omop4.eMergeCohort J 
LEFT JOIN #PreAlgVar1 J1
ON J.person_id = J1.person_id
LEFT JOIN #PreAlgVar2 J2
ON J.person_id = J2.person_id
ORDER BY person_id
;

/* *** BLOCK 3: Phenotyping **** */
IF OBJECT_ID('tempdb..#phenotype') IS NOT NULL
	DROP TABLE #phenotype;
CREATE TABLE #phenotype (
	person_id INT
	,pCase VARCHAR(50)
	,pCtrl VARCHAR(50)
	);

INSERT INTO #phenotype
SELECT DISTINCT CL.person_id /* CL has duplicate person_id */
	,ISNULL(U1.phenotype, 'NO') AS pCase
	,ISNULL(U2.phenotype, 'NO') AS pCtrl
FROM omop4.eMergeCohort CL
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C00 >0
			AND C01 >0
			)
	) U1 ON CL.person_id = U1.person_id
LEFT JOIN (
	SELECT DISTINCT person_id
		,'YES' AS phenotype
	FROM #AlgVar
	WHERE (
			C00 =0
			AND C01 =0
			AND C03 =0
			AND C04 >0
			)
	) U2 ON CL.person_id = U2.person_id;

/* *** BLOCK 4: Write each patient phenotyping result (PR) to a table *** */
IF COL_LENGTH('GenieImplementation_omop4.dbo.eMergePR', 'HeartFailureCase') IS NULL
	ALTER TABLE GenieImplementation_omop4.dbo.eMergePR ADD HeartFailureCase VARCHAR(50);
IF COL_LENGTH('GenieImplementation_omop4.dbo.eMergePR', 'HeartFailureCtrl') IS NULL
	ALTER TABLE GenieImplementation_omop4.dbo.eMergePR ADD HeartFailureCtrl VARCHAR(50);
IF COL_LENGTH('GenieImplementation_omop4.dbo.eMergePR', 'HeartFailureType') IS NULL
	ALTER TABLE GenieImplementation_omop4.dbo.eMergePR ADD HeartFailureType VARCHAR(10);

UPDATE GenieImplementation_omop4.dbo.eMergePR
SET HeartFailureCase = (
		SELECT pCase
		FROM #phenotype
		WHERE #phenotype.person_id = GenieImplementation_omop4.dbo.eMergePR.person_id
		)
	,HeartFailureCtrl = (
		SELECT pCtrl
		FROM #phenotype
		WHERE #phenotype.person_id = GenieImplementation_omop4.dbo.eMergePR.person_id
		)
	,HeartFailureType = (
		SELECT 
		CASE WHEN #AlgVar.person_id IN (SELECT DISTINCT person_id 
				FROM #phenotype WHERE #phenotype.pCase = 'YES') AND C02 IN ('1','2') THEN C02
			WHEN #AlgVar.person_id IN (SELECT DISTINCT person_id 
				FROM #phenotype WHERE #phenotype.pCase = 'YES') AND (C02 IS NULL OR C02 NOT IN ('1','2')) THEN '0'
			ELSE NULL END AS hfType
		FROM #AlgVar
		WHERE #AlgVar.person_id = GenieImplementation_omop4.dbo.eMergePR.person_id 
		);

/* *** BLOCK 5: Get count for each phenotype *** */
SELECT U1.CaseCnt
	,U2.CtrlCnt
FROM (
	SELECT Count(DISTINCT person_id) AS CaseCnt
	FROM #phenotype
	WHERE pCase = 'YES'
	) U1
	,(
		SELECT Count(DISTINCT person_id) AS CtrlCnt
		FROM #phenotype
		WHERE pCtrl = 'YES'
		) U2;


/* *** For phekb submission *** */
SELECT person_id AS eMERGE_ID
	,CASE WHEN pCase = 'YES' THEN 'C49152'
		WHEN pCtrl = 'YES' THEN 'C28143'
		WHEN pCase = 'NO' AND pCtrl = 'NO' THEN 'NO' END AS Case_Control
FROM #phenotype
ORDER BY eMERGE_ID
;

